@@ -1,13 +1,16 @@
 # -*- coding: utf-8 -*-
 
 import os
+from io import BytesIO
 
-from mutagen._compat import cBytesIO
-from mutagen.aiff import AIFF, AIFFInfo, delete, IFFFile, IFFChunk
-from mutagen.aiff import error as AIFFError
+from mutagen.aiff import AIFF, AIFFInfo, delete, AIFFFile, AIFFChunk
+from mutagen.aiff import error as AIFFError, read_float
+from mutagen._iff import error as IffError
 
 from tests import TestCase, DATA_DIR, get_temp_copy
 
+import pytest
+
 
 class TAIFF(TestCase):
     silence_1 = os.path.join(DATA_DIR, '11k-1ch-2s-silence.aif')
@@ -32,6 +35,13 @@ class TAIFF(TestCase):
         self.aiff_4 = AIFF(self.silence_4)
         self.aiff_5 = AIFF(self.silence_5)
 
+    def test_read_float(self):
+        assert read_float(b'@\x0b\xfa\x00\x00\x00\x00\x00\x00\x00') == 8000.0
+        with pytest.raises(OverflowError):
+            read_float(b"\xfa\x00\x00\xfa\x00\x00\x00\x00\x00\x00")
+        with pytest.raises(OverflowError):
+            read_float(b"\x7f\xff\x00\xfa\x00\x00\x00\x00\x00\x00")
+
     def test_channels(self):
         self.failUnlessEqual(self.aiff_1.info.channels, 1)
         self.failUnlessEqual(self.aiff_2.info.channels, 2)
@@ -60,16 +70,22 @@ class TAIFF(TestCase):
         self.failUnlessEqual(self.aiff_4.info.sample_rate, 8000)
         self.failUnlessEqual(self.aiff_5.info.sample_rate, 8000)
 
+    def test_bits_per_sample(self):
+        self.failUnlessEqual(self.aiff_1.info.bits_per_sample, 16)
+        self.failUnlessEqual(self.aiff_2.info.bits_per_sample, 16)
+        self.failUnlessEqual(self.aiff_3.info.bits_per_sample, 16)
+        self.failUnlessEqual(self.aiff_4.info.bits_per_sample, 16)
+        self.failUnlessEqual(self.aiff_5.info.bits_per_sample, 16)
+
     def test_sample_size(self):
-        self.failUnlessEqual(self.aiff_1.info.sample_size, 16)
-        self.failUnlessEqual(self.aiff_2.info.sample_size, 16)
-        self.failUnlessEqual(self.aiff_3.info.sample_size, 16)
-        self.failUnlessEqual(self.aiff_4.info.sample_size, 16)
-        self.failUnlessEqual(self.aiff_5.info.sample_size, 16)
+        for test in [self.aiff_1, self.aiff_2, self.aiff_3, self.aiff_4,
+                     self.aiff_5]:
+            info = test.info
+            self.failUnlessEqual(info.sample_size, info.bits_per_sample)
 
     def test_notaiff(self):
         self.failUnlessRaises(
-            AIFFError, AIFF, os.path.join(DATA_DIR, 'empty.ofr'))
+            IffError, AIFF, os.path.join(DATA_DIR, 'empty.ofr'))
 
     def test_pprint(self):
         self.failUnless(self.aiff_1.pprint())
@@ -138,7 +154,7 @@ class TAIFF(TestCase):
 
     def test_corrupt_tag(self):
         with open(self.filename_1, "r+b") as h:
-            chunk = IFFFile(h)[u'ID3']
+            chunk = AIFFFile(h)[u'ID3']
             h.seek(chunk.data_offset)
             h.seek(4, 1)
             h.write(b"\xff\xff")
@@ -159,7 +175,7 @@ class TAIFF(TestCase):
         self.assertEqual(AIFF(self.filename_1).tags._padding, 100)
 
         tags = AIFF(self.filename_1)
-        self.assertRaises(AIFFError, tags.save, padding=lambda x: -1)
+        self.assertRaises(IffError, tags.save, padding=lambda x: -1)
 
     def tearDown(self):
         os.unlink(self.filename_1)
@@ -169,27 +185,27 @@ class TAIFF(TestCase):
 class TAIFFInfo(TestCase):
 
     def test_empty(self):
-        fileobj = cBytesIO(b"")
-        self.failUnlessRaises(AIFFError, AIFFInfo, fileobj)
+        fileobj = BytesIO(b"")
+        self.failUnlessRaises(IffError, AIFFInfo, fileobj)
 
 
-class TIFFFile(TestCase):
+class TAIFFFile(TestCase):
     has_tags = os.path.join(DATA_DIR, 'with-id3.aif')
     no_tags = os.path.join(DATA_DIR, '8k-1ch-1s-silence.aif')
 
     def setUp(self):
         self.file_1 = open(self.has_tags, 'rb')
-        self.iff_1 = IFFFile(self.file_1)
+        self.iff_1 = AIFFFile(self.file_1)
         self.file_2 = open(self.no_tags, 'rb')
-        self.iff_2 = IFFFile(self.file_2)
+        self.iff_2 = AIFFFile(self.file_2)
 
         self.tmp_1_name = get_temp_copy(self.has_tags)
         self.file_1_tmp = open(self.tmp_1_name, 'rb+')
-        self.iff_1_tmp = IFFFile(self.file_1_tmp)
+        self.iff_1_tmp = AIFFFile(self.file_1_tmp)
 
         self.tmp_2_name = get_temp_copy(self.no_tags)
         self.file_2_tmp = open(self.tmp_2_name, 'rb+')
-        self.iff_2_tmp = IFFFile(self.file_2_tmp)
+        self.iff_2_tmp = AIFFFile(self.file_2_tmp)
 
     def tearDown(self):
         self.file_1.close()
@@ -210,10 +226,10 @@ class TIFFFile(TestCase):
         self.failUnless(u'SSND' in self.iff_2)
 
     def test_is_chunks(self):
-        self.failUnless(isinstance(self.iff_1[u'FORM'], IFFChunk))
-        self.failUnless(isinstance(self.iff_1[u'COMM'], IFFChunk))
-        self.failUnless(isinstance(self.iff_1[u'SSND'], IFFChunk))
-        self.failUnless(isinstance(self.iff_1[u'ID3'], IFFChunk))
+        self.failUnless(isinstance(self.iff_1[u'FORM'], AIFFChunk))
+        self.failUnless(isinstance(self.iff_1[u'COMM'], AIFFChunk))
+        self.failUnless(isinstance(self.iff_1[u'SSND'], AIFFChunk))
+        self.failUnless(isinstance(self.iff_1[u'ID3'], AIFFChunk))
 
     def test_chunk_size(self):
         self.failUnlessEqual(self.iff_1[u'FORM'].size, 17096)
@@ -226,9 +242,9 @@ class TIFFFile(TestCase):
     def test_FORM_chunk_resize(self):
         self.iff_1_tmp[u'FORM'].resize(17000)
         self.failUnlessEqual(
-            IFFFile(self.file_1_tmp)[u'FORM'].data_size, 17000)
-        self.iff_2_tmp[u'FORM'].resize(0)
-        self.failUnlessEqual(IFFFile(self.file_2_tmp)[u'FORM'].data_size, 0)
+            AIFFFile(self.file_1_tmp)[u'FORM'].data_size, 17000)
+        self.iff_2_tmp[u'FORM'].resize(4)
+        self.failUnlessEqual(AIFFFile(self.file_2_tmp)[u'FORM'].data_size, 4)
 
     def test_child_chunk_resize(self):
         self.iff_1_tmp[u'ID3'].resize(128)
@@ -237,27 +253,85 @@ class TIFFFile(TestCase):
         id3.write(b"\xff" * 128)
         self.assertEqual(id3.read(), b"\xff" * 128)
 
-        self.failUnlessEqual(IFFFile(self.file_1_tmp)[u'ID3'].data_size, 128)
+        self.failUnlessEqual(AIFFFile(self.file_1_tmp)[u'ID3'].data_size, 128)
         self.failUnlessEqual(
-            IFFFile(self.file_1_tmp)[u'FORM'].data_size, 16182)
+            AIFFFile(self.file_1_tmp)[u'FORM'].data_size, 16182)
 
     def test_chunk_delete(self):
         del self.iff_1_tmp[u'ID3']
         self.failIf(u'ID3' in self.iff_1_tmp)
-        self.failIf(u'ID3' in IFFFile(self.file_1_tmp))
-        self.failUnlessEqual(IFFFile(self.file_1_tmp)[u'FORM'].size, 16054)
+        self.failIf(u'ID3' in AIFFFile(self.file_1_tmp))
+        self.failUnlessEqual(AIFFFile(self.file_1_tmp)[u'FORM'].size, 16054)
         del self.iff_2_tmp[u'SSND']
         self.failIf(u'SSND' in self.iff_2_tmp)
-        self.failIf(u'SSND' in IFFFile(self.file_2_tmp))
-        self.failUnlessEqual(IFFFile(self.file_2_tmp)[u'FORM'].size, 38)
+        self.failIf(u'SSND' in AIFFFile(self.file_2_tmp))
+        self.failUnlessEqual(AIFFFile(self.file_2_tmp)[u'FORM'].size, 38)
 
     def test_insert_chunk(self):
         self.iff_2_tmp.insert_chunk(u'ID3')
 
-        new_iff = IFFFile(self.file_2_tmp)
+        new_iff = AIFFFile(self.file_2_tmp)
         self.failUnless(u'ID3' in new_iff)
-        self.failUnless(isinstance(new_iff[u'ID3'], IFFChunk))
+        self.failUnless(isinstance(new_iff[u'ID3'], AIFFChunk))
         self.failUnlessEqual(new_iff[u'FORM'].size, 16062)
         self.failUnlessEqual(new_iff[u'FORM'].data_size, 16054)
         self.failUnlessEqual(new_iff[u'ID3'].size, 8)
         self.failUnlessEqual(new_iff[u'ID3'].data_size, 0)
+
+    def test_insert_padded_chunks(self):
+        padded = self.iff_2_tmp.insert_chunk(u'TST1')
+        unpadded = self.iff_2_tmp.insert_chunk(u'TST2')
+        # The second chunk needs no padding
+        unpadded.resize(4)
+        self.failUnlessEqual(4, unpadded.data_size)
+        self.failUnlessEqual(0, unpadded.padding())
+        self.failUnlessEqual(12, unpadded.size)
+        # Resize the first chunk so it needs padding
+        padded.resize(3)
+        self.failUnlessEqual(3, padded.data_size)
+        self.failUnlessEqual(1, padded.padding())
+        self.failUnlessEqual(12, padded.size)
+        self.failUnlessEqual(padded.offset + padded.size, unpadded.offset)
+        # Verify the padding byte gets written correctly
+        self.file_2_tmp.seek(padded.data_offset)
+        self.file_2_tmp.write(b'ABCD')
+        padded.write(b'ABC')
+        self.file_2_tmp.seek(padded.data_offset)
+        self.failUnlessEqual(b'ABC\x00', self.file_2_tmp.read(4))
+        # Verify the second chunk got not overwritten
+        self.file_2_tmp.seek(unpadded.offset)
+        self.failUnlessEqual(b'TST2', self.file_2_tmp.read(4))
+
+    def test_delete_padded_chunks(self):
+        iff_file = self.iff_2_tmp
+        iff_file.insert_chunk(u'TST')
+        # Resize to odd length, should insert 1 padding byte
+        iff_file[u'TST'].resize(3)
+        # Insert another chunk after the first one
+        iff_file.insert_chunk(u'TST2')
+        iff_file[u'TST2'].resize(2)
+        self.failUnlessEqual(iff_file[u'FORM'].size, 16076)
+        self.failUnlessEqual(iff_file[u'FORM'].data_size, 16068)
+        self.failUnlessEqual(iff_file[u'TST'].size, 12)
+        self.failUnlessEqual(iff_file[u'TST'].data_size, 3)
+        self.failUnlessEqual(iff_file[u'TST'].data_offset, 16062)
+        self.failUnlessEqual(iff_file[u'TST2'].size, 10)
+        self.failUnlessEqual(iff_file[u'TST2'].data_size, 2)
+        self.failUnlessEqual(iff_file[u'TST2'].data_offset, 16074)
+        # Delete the odd chunk
+        iff_file.delete_chunk(u'TST')
+        self.failUnlessEqual(iff_file[u'FORM'].size, 16064)
+        self.failUnlessEqual(iff_file[u'FORM'].data_size, 16056)
+        self.failUnlessEqual(iff_file[u'TST2'].size, 10)
+        self.failUnlessEqual(iff_file[u'TST2'].data_size, 2)
+        self.failUnlessEqual(iff_file[u'TST2'].data_offset, 16062)
+        # Reloading the file should give the same results
+        new_iff_file = AIFFFile(self.file_2_tmp)
+        self.failUnlessEqual(new_iff_file[u'FORM'].size,
+                             iff_file[u'FORM'].size)
+        self.failUnlessEqual(new_iff_file[u'TST2'].size,
+            iff_file[u'TST2'].size)
+        self.failUnlessEqual(new_iff_file[u'TST2'].data_size,
+            iff_file[u'TST2'].data_size)
+        self.failUnlessEqual(new_iff_file[u'TST2'].data_offset,
+            iff_file[u'TST2'].data_offset)
